# To reproduce Figure 5 in the associated Physiome paper,
# execute this script from the command line:
#
#   cd [PathToThisFile]
#   [PathToOpenCOR]/pythonshell Figure5.py

import matplotlib

matplotlib.use('agg')

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.patches import Rectangle

import opencor as opencor

# different values for y_shift
y_shift = [-15, -10, -5, 0, 5, 10, 15]
t = ["time"]
V_m = {}

# load the reference model
simulation = opencor.open_simulation("HumanSAN_Fabbri_Fantini_Wilders_Severi_2017.sedml")
data = simulation.data()
data.set_ending_point(1.9)
data.set_point_interval(0.001)

V_odee = []
i_tott = []
for y in y_shift:
    # reset everything in case we are running interactively and have existing results
    simulation.reset(True)
    simulation.clear_results()
    data.constants()["i_f/i_f_y_gate/y_shift"] = y
    simulation.run()
    ds = simulation.results().data_store()
    V_m[str(y)] = ds.voi_and_variables()["Membrane/V"].values()
    V_odee.append(V_m[str(y)])
    V_m["i_tot_{}".format(str(y))] = ds.voi_and_variables()["Membrane/i_tot"].values()
    i_tott.append(V_m["i_tot_{}".format(str(y))])

simulation.reset(True)

Time = {}

simulation.run()
ds = simulation.results().data_store()
Time[t[0]] = ds.voi_and_variables()["environment/time"].values()

V_m.update(Time)

simulation.reset(True)


def _initialize_figure(size: tuple):
    plt.figure(figsize=size)


def _curation_plot(sub, x_variable, y_variable, c, m, ms, ls, lb, lw,
                   yl, xl, yt, xt, xlable, ylable, title, grid=False):
    """

    :param sub: number of subplots.
    :param x_variable: values of x axis.
    :param y_variable: values of y axis.
    :param c: colour.
    :param m: marker type.
    :param ms: marker size.
    :param ls: line style.
    :param lb: label.
    :param lw: line width.
    :param grid: True or False
    :return:
    """
    plt.subplot(sub[0], sub[1], sub[2])
    plt.plot(x_variable, y_variable, c, linestyle=ls, marker=m, markersize=ms, label=lb, linewidth=lw)
    if grid:
        plt.grid()
    plt.ylim(yl[0], yl[1])
    plt.yticks(np.arange(yt[0], yt[1], yt[2]))
    plt.xlabel(xlable, fontsize=16)
    plt.tick_params(axis='both', labelsize=14)
    plt.ylabel(ylable, fontsize=16)
    plt.title(title, loc='left', y=1.05, x=-0.06, fontsize='20')


def plot_cl(x, y):
    _curation_plot((2, 2, 1), x, y, 'navy', 'D', '14', '', '', 3, (400, 1200), None, (400, 1300, 200), None,
                   'y$_{\infty}$ shift', 'CL (ms)', 'A', True)


def plot_ddr(x, y):
    _curation_plot((2, 2, 2), x, y, 'navy', 'D', '14', '', '', 3, (20, 80), None, (20, 90, 20), None,
                   'y$_{\infty}$ shift', 'DDR$_{100}$ (mV/s)', 'B', True)


def plot_mdp(x, y):
    _curation_plot((2, 2, 3), x, y, 'navy', 'D', '14', '', '', 3, (-65, -50), None, (-65, -45, 5), None,
                   'y$_{\infty}$ shift', 'MDP (mV)', 'C', True)


def plot_apd90(x, y):
    _curation_plot((2, 2, 4), x, y, 'navy', 'D', '14', '', '', 3, (150, 180), None, (150, 190, 10), None,
                   'y$_{\infty}$ shift', 'APD$_{90}$ (ms)', 'C', True)


def compute_apd90(i_tot, V_ode):
    top = []
    for i in range(1, len(i_tot)):
        if (-i_tot[i] >= 0.5) and (-i_tot[i - 1] <= 0.5):
            top.append(i)
    # min
    mdp = []
    for i in range(1, len(V_ode) - 1):
        if (V_ode[i] < V_ode[i - 1]) and (V_ode[i] < V_ode[i + 1]):
            mdp.append(i)

    # max
    ap = []
    for i in range(1, len(V_ode) - 1):
        if (V_ode[i] > V_ode[i - 1]) and (V_ode[i] > V_ode[i + 1]):
            ap.append(i)

    apd = []
    for i in range(len(top)):
        if i < len(mdp) and mdp[i] > top[i]:
            j = V_ode[top[i]:mdp[i]][
                V_ode[top[i]:mdp[i]] >= V_ode[mdp[i]] + (1 - 90 / 100) * (max(V_ode[top[i]:mdp[i]]) - V_ode[mdp[i]])]
            j = list(np.where(V_ode == j[-1]))[-1]

            t = j - top[i]
            apd.append(t)
        elif i + 1 <= len(mdp) and mdp[i] < top[i]:
            j = V_ode[top[i]:mdp[i]][
                V_ode[top[i]:mdp[i + 1]] >= V_ode[mdp[i + 1]] + (1 - 90 / 100) * (
                        max(V_ode[top[i]:mdp[i + 1]]) - V_ode[mdp[i + 1]])]
            j = list(np.where(V_ode == j[-1]))[-1]
            t = j - top[i]
            apd.append(t)

    return apd


if __name__ == '__main__':
    y_shift = [-15, -10, -5, 0, 5, 10, 15]

    _initialize_figure((14, 14))

    """ Plotting CL """
    cl = []
    for i in range(0, 7):
        cl.append(((np.where(V_odee[i] == V_odee[i][700:1500].min()))[0]) -
                  (np.where(V_odee[i] == V_odee[i][200:700].min()))[0])

    plot_cl(y_shift, cl)

    """ Plotting DDR """
    DDR = []
    for i in range(0, 7):
        A = V_odee[i][200:700].min()
        end = (((np.where(V_odee[i] == V_odee[i][200:700].min()))[0]) + 100)
        B = V_odee[i][end]
        DDR.append((B - A) * 10)

    plot_ddr(y_shift, DDR)

    """ Plotting MDP """
    MDP = []
    for i in range(0, 7):
        A = (V_odee[i][200:600]).min()
        MDP.append(A)
    plot_mdp(y_shift, MDP)

    """ Plotting ADP90 """
    apd90 = []
    for i in range(0, 7):
        i_tot = i_tott[i] * 1000 / 57
        v_ode = V_odee[i]
        result = compute_apd90(i_tot, v_ode)
        mean = sum(result) / len(result)
        apd90.append(mean)
    plot_apd90(y_shift, apd90)

    plt.savefig('figure5.png')
